/**
 * 缓存处理函数,封装缓存规则
 * 
 * @name ucb.Cache
 * @author Ican Cheung <zhangrq@ucweb.com>
 * @version 1.0.0.0
 * 
 */
ucb.Cache = ucb.Cache || {};

(function(cache) {
    /**
     * 错误字符串
     * 
     * @type ucb.CacheProvider
     * @const
     */
    var _provider = new ucb.CacheProvider();
    var _rollerId = 0;
    var _settings = {
        // 踢数据频率
        gcFreq: 60
    };
    
    cache.setup = function(settings) {
        _.extend(_settings, settings);
    };
    
    /**
     * 设置缓存
     * 
     * @param {string}，缓存健名
     * @param {Object}，缓存数据
     * @param {int}，过期时间，秒数
     * @param {Boolean} isLocal 是否存到本地缓存，默认存在内存中
     * @return this
     */
    cache.set = function(key, val, expire, isLocal) {
        if (_.isNumber(expire) && expire >= 1) {
            var curtime = Math.ceil(new Date().getTime() / 1000);
            val = {
                expire: (curtime + expire),
                val: val
            };
        }
        
        _provider.set(key, val, isLocal);
        
        if (!_rollerId) {
//            _rollerId = ucb.utils.Roller.add(cache.gc, 'ucb.Cache.gc', {
//                isOnly: true,
//                interval: _settings.gcFreq
//            });
        }
        return this;
    };
    
    /**
     * 读取缓存
     * 
     * @param {string}，缓存健名
     * @param {Boolean} isLocal 是否存到本地缓存
     * @return {object}，key不存在或者过期返回undefined
     */
    cache.get = function(key, isLocal) {
        
        val = _provider.get(key, isLocal);
        if (!_.isUndefined(val) && !_.isUndefined(val.expire) && !_.isNull(val.expire)) {
            var curtime = Math.ceil(new Date().getTime() / 1000);
            
            if (val.expire - curtime >= 1) {
                val = val.val;
            } else {
                _provider.remove(key, isLocal);
                val = undefined;
            }
        }
        return val;
    };
    
    /**
     * 删除指定缓存
     * 
     * @param {string}，缓存健名
     * @param {Boolean} isLocal 是否存到本地缓存
     * @return this
     */
    cache.remove = function(key, isLocal) {
        _provider.remove(key, isLocal);
        return this;
    };
    
    /**
     * 踢出内存过期缓存
     * 
     * @return this
     */
    var _gcCache = function() {
        var mcache = _provider.cache;
        for ( var key in mcache) {
            if (!_.isUndefined(key)) {
                cache.get(key, false);
            }
        }
        return this;
    };
    
    /**
     * 踢出本地储存过期缓存
     * 
     * @return this
     */
    var _gcStorage = function() {
        var storage = _provider.storage;
        if (_.isEmpty(storage)) {
            return null;
        }
        var length = storage.length;
        for ( var i = 0; i < length; i++) {
            cache.get(storage.key(i), true);
        }
        return this;
    };
    
    /**
     * 踢出过期缓存
     * 
     * @return this
     */
    cache.gc = function() {
        _gcCache();
        _gcStorage();
        return this;
    };
})(ucb.Cache);