/**
 * 缓存器,封装存储介质
 * 
 * @name ucb.CacheProvider
 * @author Ican Cheung <zhangrq@ucweb.com>
 * @version 1.0.0.0
 * 
 */
ucb.CacheProvider = ucb.CacheProvider || function() {
    // 进程缓存数据存在这里
    this.cache = {};
    this._keyPrefix = "ucb_";
};

(function(cp) {
    
    try {
        cp.hasLocalStorage = ("localStorage" in window) && window["localStorage"] !== null;
    } catch (ex) {
        cp.hasLocalStorage = false;
    }
    if (cp.hasLocalStorage) {
        cp.prototype.storage = window.localStorage;
    }
    var storage = cp.prototype.storage;
    
    /**
     * @param {String} key 健名
     * @param {Boolean} isLocal 是否从本地缓存获取
     * @return mixed
     */
    cp.prototype.get = function(key, isLocal) {
        //alert(window.localStorage.getItem("ucb_24578"));
        if (isLocal && cp.hasLocalStorage) {
            return JSON.parse(storage.getItem(this._keyPrefix + key)) || undefined;
        } else {
            return this.cache[key] || undefined;
        }
    };
    
    /**
     * @param {String} key 健名
     * @param {Object} value 存储的值
     * @param {Boolean} isLocal 是否存到本地缓存
     * @return mixed
     */
    cp.prototype.set = function(key, value, isLocal) {
        if (isLocal && cp.hasLocalStorage) {
            storage.setItem(this._keyPrefix + key, JSON.stringify(value,function(k,v){if("doms"==k)return null;return v;}));
        } else {
            // 存到进程缓存
            this.cache[key] = value;
        }
        return value;
    };
    
    /**
     * @param {String} key 健名
     * @param {Boolean} isLocal 是否存到本地缓存
     * @return void
     */
    cp.prototype.remove = function(key, isLocal) {
        if (isLocal && cp.hasLocalStorage) {
            storage.removeItem(this._keyPrefix + key);
        }
        // 删除进程缓存
        delete this.cache[key];
    };
    
})(ucb.CacheProvider);